<?php
$title = "Set fuses";

require 'include/header.php';
require 'include/util.php';

$db = sqlite_open("avrweb.sqlite");

$part_shorthand = sqlite_single_query($db, "SELECT value FROM settings WHERE setting = 'part'");
$result = sqlite_unbuffered_query($db, "SELECT * FROM parts WHERE shorthand = '$part_shorthand'");
$part = sqlite_fetch_array($result);
$dude = avrdude($db, "-v");

function mask2shift($mask) { // calculate required value shift based on bit mask
	if($mask < 1)
		return 0; // error condition
		
	for($shift = 0; !($mask & (1<<$shift)); $shift++);
	
	return $shift;
}

// used to render a dropdown field
function render_dropdown($name, $field, $fuses, $enum) {
?>
	<li data-role="fieldcontain">
		<label for="<?= $name ?>" title="<?= $name ?>" class="select"><?= $field['caption'] ?>:</label>
		<select name="<?= $name ?>" id="<?= $name ?>" data-mini="true" onchange="update_fuses();">
		<?php foreach($enum as $value): // shifting option values done elsewhere ?>
		<?php if(($field['mask'] & $fuses[$field['fuse']]) == ($value['value'] << $field['shift'])): ?>
			<option value="<?= $value['value'] ?>" selected="selected"><?= $value['caption'] ?> (<?= str_pad(decbin($value['value']), 8, '0', STR_PAD_LEFT) ?>)</option>
		<?php else: ?>
			<option value="<?= $value['value'] ?>"><?= $value['caption'] ?> (<?= str_pad(decbin($value['value']), 8, '0', STR_PAD_LEFT) ?>)</option>
		<?php endif; ?>
		<?php endforeach; ?>
		</select>
	</li>
<?php
}

// used to render a toggle switch field
function render_toggle($name, $field, $fuses) {
?>
	<li data-role="fieldcontain">
		<label for="<?= $name ?>" title="<?= $name ?>"><?= $field['caption'] ?>:</label>
		<select name="<?= $name ?>" id="<?= $name ?>" data-role="slider" data-mini="true" onchange="update_fuses();">
		<?php if($field['mask'] & $fuses[$field['fuse']]): ?>
			<option value="0" selected="selected">Off</option>
			<option value="1">On</option>
		<?php else: ?>
			<option value="0">Off</option>
			<option value="1" selected="selected">On</option>
		<?php endif; ?>
		</select>
	</li>
<?php
}

?>

<div data-role="content">	

<!-- <?= $dude ?> -->
	
<h2><?= $part['description'] ?> (<?= $part['id'] ?>)</h2>

<?php if($dude): ?>

<?php 
$result = sqlite_unbuffered_query($db, "SELECT * FROM part_fuses WHERE part_id = " . $part['id']); 

$fuses = array(); // fuse name (HIGH/LOW/...) to value mapping
$fields = array(); // all fields as name to contents mapping
$enum_ids = array(); // enums to fetch

while($fuse = sqlite_fetch_array($result)) {
	switch(strtoupper($fuse['name'])) {
	case 'HIGH': 
		preg_match("/hfuse reads as ([0-9A-Z]{1,2})/", $dude, $matches);
		break;
	case 'EXTENDED': 
		preg_match("/efuse reads as ([0-9A-Z]{1,2})/", $dude, $matches);
		break;
	case 'LOW': 
		preg_match("/lfuse reads as ([0-9A-Z]{1,2})/", $dude, $matches);
		break;
	default:
		$matches = array();
	}
	
	$fuses[$fuse['name']] = $matches ? hexdec($matches[1]) : 0;
	
	$sql = "SELECT b.* FROM fuse_bitfields AS f INNER JOIN bitfields AS b ON " .
		"f.bitfield_id = b.id WHERE f.fuse_id = " . $fuse['id'];
		
	//echo "<h2>${fuse['name']} (${fuse['id']})</h2>\n";
	//echo "<p>$sql</p>";
	$result2 = sqlite_unbuffered_query($db, $sql);
	
	while($field = sqlite_fetch_array($result2)) {
		// There are currently no duplicate b.name values for a given part, evidenced by this simple
		// DB query returning an empty set:
		//
		// SELECT p.shorthand, b.name, COUNT(*) AS cnt FROM parts AS p 
		// INNER JOIN part_fuses AS pf ON p.id = pf.part_id 
		// INNER JOIN fuse_bitfields AS fb ON pf.id = fb.fuse_id 
		// INNER JOIN bitfields AS b ON fb.bitfield_id = b.id 
		// GROUP BY p.shorthand, b.name 
		// HAVING cnt > 1 
		// ORDER BY p.shorthand, cnt DESC
		if(array_key_exists($field['b.name'], $fields)) {
			echo "<p>Fatal error: Duplicate key names for bitfields. Exiting.</p>";
			exit;
		}
		
		$fields[$field['b.name']] = array(
			'fuse' => $fuse['name'],
			'caption' => $field['b.caption'],
			'mask' => (int)$field['b.mask'],
			'shift' => mask2shift((int)$field['b.mask']), # required shift for values
			'enum_id' => (int)$field['b.bitfield_enum_id']
		);
		
		if((int)$field['b.bitfield_enum_id']) // add to queue
			$enum_ids[] = (int)$field['b.bitfield_enum_id'];
	}
}

$enums = array();

foreach($enum_ids as $enum_id) {
	if(array_key_exists($enum_id, $enums)) // improbable
		continue; // avoid duplicates
		
	$enums[$enum_id] = array();
	
	$result = sqlite_unbuffered_query($db, "SELECT * FROM bitfield_enum_values WHERE bitfield_enum_id = $enum_id ORDER BY id");
	
	while($value = sqlite_fetch_array($result)) {
		$enums[$enum_id][] = array(
			'name' => $value['name'],
			'caption' => $value['caption'],
			'value' => (int)$value['value'],
		);
	}
}

// The following fields are shown in basic settings, if available
$basic_settings = array('CKDIV8', 'SUT_CKSEL', 'BODLEVEL');

$basic_fields = array();

// look for basic settings and add them
foreach($basic_settings as $name)
	if(array_key_exists($name, $fields))
		$basic_fields[] = $name;
		
$additional_fields = array();

// add all non-basic settings in original order
foreach($fields as $name => $field)
	if(array_search($name, $basic_settings) === FALSE)
		$additional_fields[] = $name;
		
if(!count($basic_fields)) { // leave "additional fields" out if no basic fields
	$field_groups = array(
		array(TRUE, "Settings", $additional_fields)
	);
} else {
	$field_groups = array(
		array(TRUE, "Basic Settings", $basic_fields),
		array(FALSE, "Additional Settings", $additional_fields)
	);
}
?>

<script type="text/javascript">
var fields = <?= json_encode($fields) ?>;

function update_fuses() {
	var s = [], fuses = {}, val;
	
	$.each(fields, function(index, field) { 
		if(!(field.fuse in fuses)) // initialize fuse if necessary
			fuses[field.fuse] = 0xFF;
		
		if(field.enum_id != 0) { // dropdown, needs shifting
			val = field.mask ^ (parseInt($('#' + index).val()) << field.shift);
			fuses[field.fuse] ^= val;
		} else if($('#' + index).val() == 1) { // selected toggle, set mask bit to zero
			fuses[field.fuse] ^= field.mask;
		}
	});
	
	$.each(fuses, function(name, value) {
		$('#FUSE_'+name).val('0x'+value.toString(16).toUpperCase());
	});
}
</script>

<form>

<?php
foreach($field_groups as $field_group) {
	if($field_group[0] === FALSE) { // field group collapsed by default
		echo '<div data-role="collapsible">';
		echo "<h3>$field_group[1]</h3>";
	}
	
	echo '<ul data-role="listview" data-inset="true">';
	echo '<li data-role="list-divider">' . $field_group[1] . '</li>';
	
	foreach($field_group[2] as $name) {
		$field = $fields[$name];
		if($field['enum_id']) {
			render_dropdown($name, $field, $fuses, $enums[$field['enum_id']]);
		} else {
			render_toggle($name, $field, $fuses);
		}
	}
	
	echo '</ul>';
	
	if($field_group[0] === FALSE) // field group collapsed by default
		echo '</div>';
}
?>

</form>

<!-- div data-role="collapsible">
   <h3>Fuse information</h3>

	<table>
	<tr>
	<th>Fuse</th>
	<th>Name</th>
	<th>Caption</th>
	<th>Mask</th>
	<th>Shift</th>
	<th>Enum</th>
	</tr>
	<?php foreach($fields as $name => $field): ?>
	<tr>
	<td><?= $field['fuse'] ?></td>
	<td><?= $name ?></td>
	<td><?= $field['caption'] ?></td>
	<td><?= str_pad(decbin($field['mask']), 8, '0', STR_PAD_LEFT) ?></td>
	<td><?= $field['shift'] ?></td>
	<td><?= $field['enum_id'] ?></td>
	</tr>
	<?php endforeach; ?>
	</table>
</div--> <!-- collapsible -->

<form action="flash.php" method="post">

<ul data-role="listview" data-inset="true">
	<li data-role="list-divider">Program</li>

<?php foreach($fuses as $fuse => $value): ?>
	<li data-role="fieldcontain">
		<label for="FUSE_<?= $fuse ?>"><?= $fuse ?> fuse:</label>
		<input type="text" name="FUSE_<?= $fuse ?>" id="FUSE_<?= $fuse ?>" value="0x<?= strtoupper(dechex($value)) ?>" data-mini="true"/>
	</li>
<?php endforeach; ?>

	<li class="ui-body ui-body-b">
		<fieldset class="ui-grid-a">
				<div class="ui-block-a"><a href="/" data-role="button" data-theme="d">Cancel</a></div>
				<div class="ui-block-b"><button type="submit" data-theme="b" 
				onclick="return confirm('Are you certain you want to flash these values?');">Flash</button></div>
		</fieldset>
	</li>
	
</ul>

</form>

<?php endif; ?>

<!--a href="#" data-role="button" data-inline="true" data-ajax="false" data-theme="b" onclick="update_fuses(); return false;">Update fuses</a-->

</div><!-- /content -->

<?php
sqlite_close($db);
require 'include/footer.php';
?>
